/**
* Barclays Bikes For Hire (BFH)
* Copyright (c) 2012 by Skinkers Ltd, Inc. All Rights Reserved.
* 
* bfh.model.dockingStations module
* Loads the TFL docking station data and parses is into JS objects
**/
var config = require('bfh/config/config');

var HTTP_REQUEST_TIMEOUT = 3000;

//Last time we loaded API
var lastLoad = 0;

//The last data that we loaded.
var dockingData = null;

//The HTTPClient for all requests in this model
var xhr;
	
/**
 * Loads the docking station data from TFL.
 * If sequential load requests are made within the time set in MIN_LOAD_INTERVAL
 * Then the load is ignored. This is to stop the user from hitting the server to often.
 * If a request is beyond the time limit, then any pending requests are cancelled 
 * and the new request is made. 
 */
var loadDockingStations = function()
{
	//Get time now
	var now = new Date().getTime();
	
	//Check the time diff since last load
	var timeDelta = now - lastLoad;
	
//	if ( (timeDelta <= config.minimumLoadInterval()) && lastLoad != 0)
//		return;
	
	//If we havent loaded any data yet, return the cached data from disk so we have something to plot on the map
	if(lastLoad == 0)
	{
		dockingData = loadDockingDataFromFile();
		
		if ( dockingData.stations.length>0 ) {
			Ti.App.fireEvent('app:dockingStationsLoaded'); //Not passing data in event as its too big to serialise down to the API and back, causes crashes
		}
		
	}
			
	makeHTTPRequest(now);	
	lastLoad = now;
	
}

/**
 * Resets the last loaded time so that the next request will be made.
 */
var resetLastLoadTime = function()
{
	lastLoad = 0;
}

/**
 * Makes an http request for the dock station xml. Ensures an http client exists before making the request.
 */
function makeHTTPRequest() 
{
	if(!xhr)
		createHTTPClient();
	
	//Set the timeout
    xhr.timeout = HTTP_REQUEST_TIMEOUT;

    //Abort existing call
    xhr.abort();
	
    // open the client
    xhr.open('GET', config.dockingApiUrl());
    
    	    
 	// send the data
    xhr.send();
}

/**
 * Creates an http client and assigns callbacks to load and error handlers.
 * @private
 */
function createHTTPClient()
{
	xhr = Titanium.Network.createHTTPClient();

	xhr.onload = xhrLoadHandler;
	xhr.onerror = xhrErrorHandler;
}

/**
 * HTTP Client error handler
 * @private
 */
function xhrErrorHandler() {
	var error = this.status + " " + this.responseText;
	Ti.API.error('ERROR ' + error);
	
	// more verbose than alert(), but more testable.
	var alert = Ti.UI.createAlertDialog({
		message:'Error handler fired ' + error
	});
	
	alert.show();
}

/**
 * HTTP Client loaded handler
 * Parses the response as JSON and triggers an event containg the resuls array
 * @private
 */
function xhrLoadHandler() {
	var data = null;
	
	Ti.API.info('XHR has loaded..');   
	    
	try {    
		//Suggestion from TI was to use response text, and then parse the text rather than responseXML
		//Either way appears to fail
		dockingData = parseXML( this.responseText );
		dockingData.lastPoll = lastLoad;
		
		saveDockingDataToFile(dockingData); 
	}
	catch(error) {
		Ti.API.error('error parsing docking station data. ' + error);
	}
	
	Ti.API.info('data has parsed..');  
	
	//Fire event to notify that the data has loaded, passing the data
	Ti.App.fireEvent('app:dockingStationsLoaded' ); //Not passing data in event as its too big to serialise down to the API and back, causes crashes
}

/**
 * Takes an object and saves it to disc as a json string.
 * @param data	A data object
 */
function saveDockingDataToFile(data) {
	
	if(!data)
		return;
	
	try {
		retrieveFile().write(JSON.stringify(data), false);
	}
	catch(error) {
		Ti.API.error("Failed to write data to file. Error: " + error);
	}
}

/**
 * Retrieves the Titanium File object representing the file used to store docking station data.
 * @return A File object
 * @private
 */
function retrieveFile() {
	return Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory + config.dockingDataFilename());
}

/**
 * Loads JSON data from the file system.
 * @return An object in the same form as a parsed docking station XML object.
 * @private
 */
function loadDockingDataFromFile() {
	
	var data = {
		stations:[],
		lastUpdate:null
	};
	
	var file = retrieveFile();
	var jsonData = retrieveFile().read();
	
	if (jsonData) {
		try {
			data = JSON.parse(jsonData);
		}
		catch(error) {
			data = null;
		}
	} 
	else {
		data = null;	
	}
	
	return data;
}

/**
 * Parse the TFL docking station XML
 * @param xmlDom XML DOM of the data to parse to JS
 * @return JS Array of docking station objects
 * @private
 */
function parseXML(xmlStr) {
		
	Ti.API.info('parse XML...');  
		
	var xmlDom = Ti.XML.parseString( xmlStr );	
		
	var data = {
		stations:[],
		lastUpdate:xmlDom.documentElement.getAttribute('lastUpdate')
	};
	
	//:TODO:  note SDK 1.7.x crashes parsing this XML as its a large amout of data.
	//You need to ensure its 1.8 above
	var docksXML = xmlDom.documentElement.getElementsByTagName('station');
	
	//attempt to free up memory...
	xmlDom = null;
	
	var length = docksXML.length;
	
	for (var i=0; i<length; i++) {
		
		var dock = {};
		
		var xmlNodeData = docksXML.item(i);
		
		Ti.API.info('about to convert XML node to JS node number ' + i);  
		
		if ( retrieveNodeValue('installed', xmlNodeData ) == 'true'){
		
			dock.id = retrieveNodeValue('id', xmlNodeData );
			dock.name = retrieveNodeValue('name', xmlNodeData );
			dock.lat = retrieveNodeValue('lat', xmlNodeData );
			dock.lng = retrieveNodeValue('long', xmlNodeData );
			dock.nbBikes = retrieveNodeValue('nbBikes', xmlNodeData );
			dock.nbDocks = retrieveNodeValue('nbDocks', xmlNodeData );
			dock.nbEmptyDocks = retrieveNodeValue('nbEmptyDocks', xmlNodeData );
		}
		
		data.stations.push(dock);
	}
	
	return data;
		
}

/**
 * Checks for the existance of an XML node, and returns the node value, or null
 * @param String the name of the node to retrieve
 * @param XMLNode to retrieve node from
 * @return String -the node value, or null if it does not exist
 * @private
 */
function retrieveNodeValue( nodeName, xmlNode) {
	
	var value='';
	
	try {
		var element = xmlNode.getElementsByTagName( nodeName ); 
		if ( element ) {
			var node = element.item(0); 
			if (node) {
				value = node.text;
			}
		}
	}
	catch(e){
		Ti.API.error('Retreiving Node name ' + nodeName + ' failed on XML object. XML object is ' + xmlNode);
	}
	
	return value;
}


/**
 * Returns the last docking data object that was loaded.
 * @return dockingData Object, the docking data object.
 */
var retrieveCurrentDockingData=function() {
	return dockingData;
}

//Publicise methods
exports.loadDockingStations = loadDockingStations;
exports.retrieveCurrentDockingData = retrieveCurrentDockingData;
exports.resetLastLoadTime = resetLastLoadTime;





